# -*- coding: utf-8 -*-

# authorisation.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Behaviour for authorisation of OpenID requests
"""

import logging

# Get the Python logging instance for this module
_logger = logging.getLogger("gracie.authorisation")


class ConsumerAuthStore(object):
    """ Storage for consumer request authorisations """

    def __init__(self):
        """ Set up a new instance """
        self._authorisations = dict()

    def is_authorised(self, auth_tuple):
        """ Report authorisation of an (identity, known_root) tuple """
        is_authorised = self._authorisations.get(
            auth_tuple, False)
        return is_authorised

    def store_authorisation(self, auth_tuple, status):
        """ Store an authorisation status """
        self._authorisations[auth_tuple] = status

    def remove_authorisation(self, auth_tuple):
        """ Remove an authorisation status """
        if auth_tuple in self._authorisations:
            del self._authorisations[auth_tuple]




#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_httpresponse.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for httpresponse module
"""

import sys
from StringIO import StringIO

import scaffold
from scaffold import Mock

from gracie import httpresponse


class Stub_Response(object):
    """ Stub class for Response """

    def __init__(self, header, data=None):
        """ Set up a new instance """
        self.header = header
        self.data = data

class Test_ResponseHeader(scaffold.TestCase):
    """ Test cases for ResponseHeader class """

    def setUp(self):
        """ Set up test fixtures """

        self.header_class = httpresponse.ResponseHeader

        self.valid_headers = {
            'simple': dict(
                code = 200,
            ),
            'ok': dict(
                code = 200,
            ),
            'ok-protocol': dict(
                code = 200,
                protocol = "HTTP/1.1",
            ),
            'content-type-bogus': dict(
                code = 200,
                content_type = "BoGuS",
            ),
        }

        for key, params in self.valid_headers.items():
            args = params.get('args', dict())
            code = params['code']
            args['code'] = code
            protocol = params.get('protocol')
            if protocol is not None:
                args['protocol'] = protocol
            content_type = params.get('content_type')
            if content_type is not None:
                args['content_type'] = content_type
            params['args'] = args
            instance = self.header_class(**args)
            params['instance'] = instance

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_headers
        )

    def test_instantiate(self):
        """ New ResponseHeader instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_code_as_specified(self):
        """ ResponseHeader should have specified status code """
        for key, params in self.iterate_params():
            code = params['code']
            instance = params['instance']
            self.failUnlessEqual(code, instance.code)

    def test_protocol_default_http_1_0(self):
        """ ResponseHeader protocol should default to HTTP/1.0 """
        params = self.valid_headers['simple']
        instance = params['instance']
        protocol = "HTTP/1.0"
        self.failUnlessEqual(protocol, instance.protocol)

    def test_protocol_as_specified(self):
        """ ResponseHeader should have specified response protocol """
        for key, params in self.iterate_params():
            protocol = params.get('protocol')
            if protocol is None:
                continue
            instance = params['instance']
            self.failUnlessEqual(protocol, instance.protocol)

    def test_content_type_default_xhtml(self):
        """ ResponseHeader should default to Content-Type of XHTML """
        params = self.valid_headers['simple']
        instance = params['instance']
        expect_field =("Content-Type", "application/xhtml+xml")
        self.failUnless(expect_field in instance.fields)

    def test_content_type_as_specified(self):
        """ ResponseHeader should have specified Content-Type field """
        for key, params in self.iterate_params():
            content_type = params.get('content_type')
            expect_field = ("Content-Type", content_type)
            if content_type is None:
                continue
            instance = params['instance']
            self.failUnless(expect_field in instance.fields)


class Stub_ResponseHeader(object):
    """ Stub class for response header """

    def __init__(self, code, protocol=None):
        self.code = code
        self.protocol = protocol
        self.fields = dict()

class Stub_RequestHandler(object):
    """ Stub class for BaseHTTPRequestHandler """

    def __init__(self):
        self.wfile = StringIO("")
    def send_response(self, code, message=None): pass
    def end_headers(self): pass

class Test_Response(scaffold.TestCase):
    """ Test cases for Response class """

    def setUp(self):
        """ Set up test fixtures """

        self.response_class = httpresponse.Response

        self.valid_responses = {
            'simple': dict(
                header = Stub_ResponseHeader(code = 200),
            ),
            'payload': dict(
                header = Stub_ResponseHeader(code = 200),
                data = object(),
            ),
        }

        for key, params in self.valid_responses.items():
            args = params.get('args', dict())
            header = params['header']
            args['header'] = header
            data = params.get('data')
            if data is not None:
                args['data'] = data
            params['args'] = args
            instance = self.response_class(**args)
            params['instance'] = instance

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_responses
        )

    def test_initialise(self):
        """ New Response instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_header_as_specified(self):
        """ Response should have specified header """
        for key, params in self.iterate_params():
            header = params['header']
            instance = params['instance']
            self.failUnlessEqual(header, instance.header)

    def test_data_as_specified(self):
        """ Response should have specified data """
        params = self.valid_responses['payload']
        data = params['data']
        instance = params['instance']
        self.failUnlessEqual(data, instance.data)

    def test_send_to_handler_uses_handler(self):
        """ Response.send_to_handler should use specified handler """
        self.stdout_test = StringIO("")
        stdout_prev = sys.stdout
        sys.stdout = self.stdout_test
        for key, params in self.iterate_params():
            instance = params['instance']
            handler = Mock('HTTPRequestHandler')
            instance.send_to_handler(handler)
            expect_stdout = """\
                Called HTTPRequestHandler.send_response(...)
                ...Called HTTPRequestHandler.end_headers()
                Called HTTPRequestHandler.wfile.write(...)
                Called HTTPRequestHandler.wfile.close()
                """
            self.failUnlessOutputCheckerMatch(
                expect_stdout, self.stdout_test.getvalue()
            )
        sys.stdout = stdout_prev


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_httpserver.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for httpserver module
"""

import sys
import os
from StringIO import StringIO

import scaffold
from scaffold import Mock
from test_httpresponse import Stub_ResponseHeader, Stub_Response
from test_server import stub_server_bind, make_default_opts
from test_server import (
    Stub_ConsumerAuthStore,
    Stub_ConsumerAuthStore_always_auth,
    Stub_ConsumerAuthStore_never_auth,
)
from test_gracied import Stub_GracieServer

from gracie import httpserver


class Test_net_location(scaffold.TestCase):
    """ Test cases for net_location function """

    def setUp(self):
        """ Set up test fixtures """
        self.default_port_prev = httpserver.default_port
        self.default_port_test = 9779
        httpserver.default_port = self.default_port_test

    def tearDown(self):
        """ Tear down test fixtures """
        httpserver.default_port = self.default_port_prev

    def test_combines_components_to_location(self):
        """ net_location() should combine components to net location """
        http_port = 80
        locations = {
            ("foo", None): "foo",
            ("foo", http_port): "foo",
            ("foo", self.default_port_test):
                "foo:%s" % self.default_port_test,
            ("foo", 2468): "foo:2468",
        }

        for (host, port), expect_location in locations.items():
            location = httpserver.net_location(host, port)
            self.failUnlessEqual(expect_location, location)


class Stub_HTTPRequestHandler(object):
    """ Stub class for HTTPRequestHandler """


class Test_HTTPServer(scaffold.TestCase):
    """ Test cases for HTTPServer class """

    def setUp(self):
        """ Set up test fixtures """

        self.server_class = httpserver.HTTPServer

        self.stub_handler_class = Stub_HTTPRequestHandler
        self.mock_handler_class = Mock('HTTPRequestHandler')

        self.server_bind_prev = httpserver.BaseHTTPServer.server_bind
        httpserver.BaseHTTPServer.server_bind = stub_server_bind

        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.valid_servers = {
            'simple': dict(
                address = ('', 80),
            ),
        }

        for key, params in self.valid_servers.items():
            args = params.get('args')
            address = params.get('address')
            handler_class = params.setdefault(
                'handler_class', self.stub_handler_class)
            opts = make_default_opts()
            gracie_server = params.setdefault(
                'gracie_server', Stub_GracieServer(address, opts)
            )
            if not args:
                args = dict(
                    server_address = params['address'],
                    RequestHandlerClass = params['handler_class'],
                    gracie_server = params['gracie_server']
                )
            instance = self.server_class(**args)
            params['args'] = args
            params['instance'] = instance

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_servers
        )

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        httpserver.BaseHTTPServer.server_bind = self.server_bind_prev

    def test_instantiate(self):
        """ New HTTPServer instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_version_as_specified(self):
        """ HTTPServer should have specified version string """
        params = self.valid_servers['simple']
        version_prev = httpserver.server.__version__
        version_test = Stub_GracieServer.version
        httpserver.server.__version__ = version_test
        instance = self.server_class(**params['args'])
        self.failUnlessEqual(version_test, instance.version)
        httpserver.server.__version__ = version_prev

    def test_request_handler_class_as_specified(self):
        """ HTTPServer should have specified RequestHandlerClass """
        for key, params in self.iterate_params():
            instance = params['instance']
            handler_class = params['handler_class']
            self.failUnlessEqual(
                handler_class, instance.RequestHandlerClass
            )

    def test_gracie_server_as_specified(self):
        """ HTTPServer should have specified Gracie server instance """
        for key, params in self.iterate_params():
            instance = params['instance']
            gracie_server = params['gracie_server']
            self.failUnlessEqual(
                gracie_server, instance.gracie_server
            )

    def test_serve_forever_is_callable(self):
        """ HTTPServer.serve_forever should be callable """
        self.failUnless(callable(self.server_class.serve_forever))


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



# -*- coding: utf-8 -*-

# server.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Behaviour for OpenID provider server
"""

import sys
import os
import logging
from openid.server.server import Server as OpenIDServer
from openid.store.filestore import FileOpenIDStore as OpenIDStore

from httprequest import HTTPRequestHandler
from httpserver import HTTPServer, default_host, default_port
from authservice import PamAuthService as AuthService
from authorisation import ConsumerAuthStore
from session import SessionManager

__version__ = "0.2.4"

# Get the Python logging instance for this module
_logger = logging.getLogger("gracie.server")


def remove_standard_files():
    """ Close stdin, redirect stdout & stderr to null """
    class NullDevice:
        def write(self, s):
            pass
    sys.stdin.close()
    sys.stdout = NullDevice()
    sys.stderr = NullDevice()

def become_daemon():
    """ Detach the current process and run as a daemon """
    # This technique cribbed from Chad J. Schroeder,
    # <URL:http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/278731>

    pid = os.fork()
    if pid == 0:
        # This is the child of the first fork, so we are now in the
        # background.

        # Set a new process group
        os.setsid()

        pid = os.fork()
        if pid == 0:
            # This is the parent of the new process group, and is
            # orphaned from the original parent process. Good.
            pass
        else:
            # This is the child of the second fork, so we want to exit
            # orphaning the true process to run by itself.
            os._exit(os.EX_OK)
    else:
        # This is the parent process of the first fork
        # so we want to exit, leaving only the child to run
        os._exit(os.EX_OK)

    remove_standard_files()


class GracieServer(object):
    """ Server for Gracie OpenID provider service """

    def __init__(self, socket_params, opts):
        """ Set up a new instance """
        self.version = __version__
        self.opts = opts
        self._setup_logging()
        server_address = (opts.host, opts.port)
        self.httpserver = HTTPServer(
            server_address, HTTPRequestHandler, self
        )
        self._setup_openid()
        self.auth_service = AuthService()
        self.sess_manager = SessionManager()
        self.consumer_auth_store = ConsumerAuthStore()

    def _setup_openid(self):
        """ Set up OpenID parameters """
        store = OpenIDStore(self.opts.datadir)
        self.openid_server = OpenIDServer(store)

    def __del__(self):
        _logger.info("Exiting Gracie server")

    def _setup_logging(self):
        """ Set up logging for this server """
        server_version = __version__
        _logger.info(
            "Starting Gracie server (version %(server_version)s)"
            % locals()
        )

    def serve_forever(self):
        """ Begin serving requests indefinitely """
        self.httpserver.serve_forever()



# -*- coding: utf-8 -*-

# session.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Behaviour for session management
"""

import logging
import random
import sha

# Get the Python logger instance for this module
_logger = logging.getLogger("gracie.session")


class SessionManager(object):
    """ Manage user sessions across transactions """

    def __init__(self):
        """ Set up a new instance """
        self._init_session_generator()

    def _init_session_generator(self):
        """ Initialise the session ID generator """
        self._rng = random.Random()
        self._rng.seed()
        self._sessions = dict()

    def _generate_session_id(self):
        """ Generate a unique session ID """
        randnum = self._rng.random()
        message = "%(randnum)s" % locals()
        message_hash = sha.sha(message)
        session_id = message_hash.hexdigest()
        return session_id

    def create_session(self, session=None):
        """ Create a new session for supplied session dict """
        if session is None:
            session = dict()
        session_id = self._generate_session_id()
        session['session_id'] = session_id
        self._sessions[session_id] = session
        return session_id

    def get_session(self, session_id):
        """ Get the session for specified session ID """
        session = self._sessions[session_id]
        return session

    def remove_session(self, session_id):
        """ Remove the specified session """
        del self._sessions[session_id]



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_authservice.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for authservice module
"""

import scaffold
from scaffold import Mock

from gracie import authservice


class Test_ModuleExceptions(scaffold.Test_Exception):
    """ Test cases for module exceptions """

    def setUp(self):
        """ Set up test fixtures """
        self.valid_exceptions = {
            authservice.AuthenticationError: dict(
                min_args = 2,
                types = [EnvironmentError],
            ),
        }

        super(Test_ModuleExceptions, self).setUp()

    def test_autherror_string_contains_reason(self):
        """ AuthenticationError string should contain code and reason """
        code = 42
        reason = "Naughty!"
        instance = authservice.AuthenticationError(code, reason)
        str(instance)
        self.failUnlessOutputCheckerMatch(
            "...%(code)s..." % locals(), str(instance)
        )
        self.failUnlessOutputCheckerMatch(
            "...%(reason)s..." % locals(), str(instance)
        )


class Test_BaseAuthService(scaffold.TestCase):
    """ Test cases for AuthService class """

    def setUp(self):
        """ Set up test fixtures """

        self.service_class = authservice.BaseAuthService

    def test_instantiate(self):
        """ New BaseAuthService instance should be created """
        instance = self.service_class()
        self.failUnless(instance is not None)

    def test_name_is_not_implemented(self):
        """ BaseAuthService should have name NotImplemented """
        instance = self.service_class()
        self.failUnlessEqual(NotImplemented, instance.name)

    def test_get_entry_is_not_implemented(self):
        """ BaseAuthService should not implement get_entry() """
        instance = self.service_class()
        name = "foo"
        self.failUnlessRaises(NotImplementedError,
            instance.get_entry, name
        )

    def test_authenticate_is_not_implemented(self):
        """ BaseAuthService should not implement authenticat() """
        instance = self.service_class()
        credentials = dict(
            username = "foo",
            password = "bar",
        )
        self.failUnlessRaises(NotImplementedError,
            instance.authenticate, credentials
        )


stub_entries = [
    dict(id=1000, name="fred", password="password1",
         fullname="Fred Nurk", comment="Fred Nurk"),
    dict(id=1010, name="bill", password="secret1",
         fullname="William Fosdycke", comment="William Fosdycke,,,"),
]

class Stub_AuthService(object):
    """ Stub class for AuthService classes """

    def get_entry(self, value):
        match = [e for e in stub_entries if e['name'] == value]
        if not match:
            raise KeyError("No entry for %(value)r" % locals())
        entry = match[0]
        return entry

    def authenticate(self, credentials):
        try:
            username = credentials['username']
            password = credentials['password']
            entry = self.get_entry(username)
        except KeyError, e:
            raise authservice.AuthenticationError(
                0, "User not found")
        if password != entry['password']:
            raise authservice.AuthenticationError(
                0, "Authentication failed")
        return username

class Stub_PwdModule(object):
    """ Stub class for a pwd module """

    _entries = [(e['name'], "*", e['id'], 500, e['comment'],
                 "/home/"+e['name'], "/bin/sh")
                for e in stub_entries]

    _entries_by_name = dict([(e[0], e) for e in _entries])
    _entries_by_uid = dict([(e[2], e) for e in _entries])

    def getpwnam(self, name):
        """ Get an entry by account name """
        try:
            entry = self._entries_by_name[name]
        except KeyError:
            raise KeyError("name not found: %(name)s" % locals())
        return entry

    def getpwuid(self, uid):
        """ Get an entry by account ID """
        try:
            entry = self._entries_by_uid[uid]
        except KeyError:
            raise KeyError("uid not found: %(uid)s" % locals())
        return entry

class Test_PosixAuthService(scaffold.TestCase):
    """ Test cases for PosixAuthService class """

    def setUp(self):
        """ Set up test fixtures """

        self.service_class = authservice.PosixAuthService

        self.pwd_module_prev = authservice.pwd
        self.pwd_module = Stub_PwdModule()
        authservice.pwd = self.pwd_module

    def tearDown(self):
        """ Tear down test fixtures """
        authservice.pwd = self.pwd_module_prev

    def test_instantiate(self):
        """ New PosixAuthService instance should be created """
        instance = self.service_class()
        self.failUnless(instance is not None)

    def test_get_entry_name_unknown_raises_keyerror(self):
        """ get_entry for a bogus name should raise KeyError """
        instance = self.service_class()
        name = "nosuchuser"
        self.failUnlessRaises(KeyError,
            instance.get_entry, name
        )

    def test_get_entry_name_known_returns_entry(self):
        """ get_entry for a known name should return an auth entry """
        instance = self.service_class()
        pwd_entry = self.pwd_module.getpwnam("fred")
        (name, _, uid, _, fullname, _, _) = pwd_entry
        expect_entry = dict(
            id = uid,
            name = name,
            fullname = fullname,
        )
        entry = instance.get_entry(name)
        self.failUnlessEqual(expect_entry, entry)

    def test_get_entry_strips_extra_info_from_comment(self):
        """ get_entry should strip extra info to get the fullname """
        instance = self.service_class()
        pwd_entry = self.pwd_module.getpwnam("bill")
        (name, _, uid, _, comment, _, _) = pwd_entry
        fullname = comment.split(",")[0]
        expect_entry = dict(
            id = uid,
            name = name,
            fullname = fullname,
        )
        entry = instance.get_entry(name)
        self.failUnlessEqual(expect_entry, entry)


class Stub_PamError(Exception):
    def __init__(self, code, reason):
        Exception.__init__(self, reason, code)

class Stub_PamAuth(object):
    """ Stub class for PAM authentication service """

    _entries = dict([(e['name'], e) for e in stub_entries])

    def start(self, service):
        self._items = dict()

    def get_item(self, key):
        return self._items[key]

    def set_item(self, key, value):
        self._items[key] = value

    def _converse(self, query_list):
        conversation = self._items[PAM.PAM_CONV]
        response_list = conversation(self, query_list, None)
        return response_list

    def _get_auth_entry(self, username):
        try:
            entry = self._entries[username]
        except KeyError:
            raise PAM.error(PAM.PAM_USER_UNKNOWN,
                "User %(username)s not found" % locals()
            )
        return entry

    def authenticate(self, flags=0):
        username = self._items[PAM.PAM_USER]
        entry = self._get_auth_entry(username)

        query_list = []
        query_list.append(("Password: ", PAM.PAM_PROMPT_ECHO_OFF))
        response_list = self._converse(query_list)

        auth_success = False
        if response_list is None:
            response_list = []
        for response, _ in response_list:
            if not len(response):
                continue
            password = response
            if password == entry['password']:
                query_list = [
                    ("Login successful", PAM.PAM_SUCCESS)
                ]
                self._converse(query_list)
                auth_success = True

        if not auth_success:
            raise PAM.error(PAM.PAM_AUTH_ERR, "Authentication failed")

    def acct_mgmt(self): pass

class Stub_PamModule(object):
    """ Stub class for PAM authentication module """

    error = Stub_PamError

    [
    PAM_USER,
    PAM_CONV,
    PAM_SILENT,
    PAM_DISALLOW_NULL_AUTHTOK,
    PAM_PROMPT_ECHO_ON,
    PAM_PROMPT_ECHO_OFF,
    PAM_ERROR_MSG,
    PAM_TEXT_INFO,

    PAM_ABORT,
    PAM_AUTH_ERR,
    PAM_CRED_INSUFFICIENT,
    PAM_AUTHINFO_UNVAIL,
    PAM_MAXTRIES,
    PAM_USER_UNKNOWN,

    PAM_SUCCESS,
    ] = range(15)

    def pam(self):
        """ Return a handle to the authentication system """
        authenticator = Stub_PamAuth()
        return authenticator

stub_pam_module = Stub_PamModule()
PAM = stub_pam_module

class Test_PamAuthService(scaffold.TestCase):
    """ Test cases for PamAuthService class """

    def setUp(self):
        """ Set up test fixtures """

        self.service_class = authservice.PamAuthService

        self.pwd_module_prev = authservice.pwd
        self.pam_module_prev = authservice.PAM
        self.pwd_module = Stub_PwdModule()
        self.pam_module = stub_pam_module
        authservice.pwd = self.pwd_module
        authservice.PAM = self.pam_module

    def tearDown(self):
        """ Tear down test fixtures """
        authservice.pwd = self.pwd_module_prev
        authservice.PAM = self.pam_module_prev

    def test_instantiate(self):
        """ New PosixAuthService instance should be created """
        instance = self.service_class()
        self.failUnless(instance is not None)

    def test_authenticate_unknown_user_raises_autherror(self):
        """ Auth for unknown user should raise AuthenticationError """
        instance = self.service_class()
        credentials = dict(
            username = "bogus",
            password = "bogus",
        )
        self.failUnlessRaises(authservice.AuthenticationError,
            instance.authenticate, credentials
        )

    def test_authenticate_bad_creds_returns_false(self):
        """ Auth for bad credentials should raise AuthenticationError """
        instance = self.service_class()
        credentials = dict(
            username = "fred",
            password = "bogus",
        )
        self.failUnlessRaises(authservice.AuthenticationError,
            instance.authenticate, credentials
        )

    def test_authenticate_good_creds_returns_username(self):
        """ Authentication for good credentials should return username """
        instance = self.service_class()
        credentials = dict(
            username = "fred",
            password = "password1",
        )
        username = credentials['username']
        result = instance.authenticate(credentials)
        self.failUnlessEqual(username, result)


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



# -*- coding: utf-8 -*-

# _skel.py *** FIXME ***
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" *** FIXME ***
"""



# -*- coding: utf-8 -*-

# http_response.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Utility module for HTTP response handling
"""


from BaseHTTPServer import BaseHTTPRequestHandler
response_map = BaseHTTPRequestHandler.responses

response_codes = dict()
for code, (reason, explain) in response_map.items():
    key = reason.lower().replace(" ", "_")
    response_codes[key] = code

content_type_xhtml = "application/xhtml+xml"


class ResponseHeader(object):
    """ Encapsulation of an HTTP response header """

    def __init__(self, code,
        protocol="HTTP/1.0", content_type=content_type_xhtml
    ):
        """ Set up a new instance """
        self.code = code
        self.protocol = protocol
        self.fields = []
        self.fields.append(("Content-Type", content_type))

class Response(object):
    """ Encapsulation for an HTTP response """

    def __init__(self, header, data=None):
        """ Set up a new instance """
        self.header = header
        self.data = data

    def send_to_handler(self, handler):
        """ Send this response via a request handler """
        handler.send_response(self.header.code)
        for key, value in self.header.fields:
            handler.send_header(key, value)
        handler.end_headers()
        handler.wfile.write(self.data)
        handler.wfile.close()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_pagetemplate.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for pagetemplate module
"""

from string import Template
import textwrap

import scaffold
from scaffold import Mock

from gracie import pagetemplate


class Mixin_PageTemplateFixture(object):
    """ Mix-in class for page template fixtures """

    def setUp(self):
        """ Set up test fixtures """

        pagetemplate.page_template = Template(
            textwrap.dedent("""\
            Page {
                Coding: $character_encoding
                Title: $page_title
                $page_body
            }""")
        )

        pagetemplate.body_template = Template(
            textwrap.dedent("""\
            Body {
                Title: $page_title
                Auth entry: $auth_entry
                Content: $page_content
            }""")
        )

    def tearDown(self):
        """ Tear down test fixtures """


class Test_Page(scaffold.TestCase,
                Mixin_PageTemplateFixture):
    """ Test cases for Page class """

    def setUp(self):
        """ Set up test fixtures """

        self.page_class = pagetemplate.Page

        Mixin_PageTemplateFixture.setUp(self)

        self.valid_pages = {
            'simple': dict(
            ),
            'welcome': dict(
                title = "Welcome",
                content = "Lorem ipsum dolor sic amet",
            ),
            'result': dict(
                title = "Result",
                content = "Here is your result: $result.",
                values = dict(
                    result = "thribble",
                ),
            ),
            'authenticated': dict(
                title = "Logged in",
                content = "Lorem ipsum dolor sic amet",
                auth_entry = dict(
                    id = 1010,
                    name = "fred",
                    fullname = "Fred Nurk",
                ),
            ),
        }

        for key, params in self.valid_pages.items():
            args = params.get('args')
            if args is None:
                args = dict()
            title = params.get('title')
            if title is not None:
                args.update(dict(title=title))
            params['args'] = args
            instance = self.page_class(**args)
            content = params.get('content')
            if content is not None:
                instance.content = content
            values = params.get('values', dict())
            if 'auth_entry' not in values:
                values.update({'auth_entry': None})
            values.update(dict(
                server_version = "FooBar v0.0",
                server_location = "frobnitz:9779",
                root_url = "/",
                server_url = "/openidserver",
                login_url = "/login",
                logout_url = "/logout",
            ))
            instance.values.update(values)
            params['instance'] = instance

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_pages
        )

    def test_instantiate(self):
        """ New Page instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_title_as_specified(self):
        """ Page title should be as specified """
        for key, params in self.iterate_params():
            title = params.get('title')
            if not title:
                continue
            instance = params['instance']
            self.failUnlessEqual(title, instance.title)

    def test_values_as_specified(self):
        """ Page values should be as specified """
        for key, params in self.iterate_params():
            values = params.get('values')
            if not values:
                continue
            instance = params['instance']
            self.failUnlessEqual(values, instance.values)

    def test_serialise_uses_template(self):
        """ Page.serialise should use provided template """
        params = self.valid_pages['welcome']
        instance = params['instance']
        expect_data = """\
            Page {
                Coding: ...
                Title: ...
                Body {
                Title: ...
                Auth entry: ...
                Content: ...
            }
            }"""
        page_data = instance.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)

    def test_serialise_substitutes_values(self):
        """ Page.serialise should substitute values into template """
        params = self.valid_pages['result']
        instance = params['instance']
        title = params['title']
        content = params['content']
        values = params['values']
        auth_entry = values['auth_entry']
        expect_content = Template(content).substitute(values)
        expect_data = """\
            Page {
                Coding: utf-8
                Title: %(title)s
                Body {
                Title: %(title)s
                Auth entry: %(auth_entry)s
                Content: %(expect_content)s
            }
            }""" % locals()
        page_data = instance.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)


class Test_PageTemplates(scaffold.TestCase):
    """ Test cases for individual page templates """

    def test_internal_error_page_contains_message(self):
        """ Internal Error page should contain specified message """
        message = "Bad stuff happened"
        page = pagetemplate.internal_error_page(message)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...%(message)s...}" % locals(), page_data
        )

    def test_url_not_found_page_contains_url(self):
        """ Resulting page should contain the referent URL """
        url = "/flim/flam/flom"
        page = pagetemplate.url_not_found_page(url)
        expect_data = "...%(url)s..." % locals()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)

    def test_protocol_error_page_contains_message(self):
        """ Protocol Error page should contain specified message """
        message = "Bad stuff happened"
        page = pagetemplate.protocol_error_page(message)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...%(message)s...}" % locals(), page_data
        )

    def test_about_site_page_returns_page(self):
        """ About Site page should return page """
        page = pagetemplate.about_site_view_page()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...}", page_data
        )

    def test_identity_user_not_found_page_contains_name(self):
        """ Resulting page should contain the referent user name """
        name = "fred"
        page = pagetemplate.identity_user_not_found_page(name)
        expect_data = "...%(name)s..." % locals()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(expect_data, page_data)

    def test_identity_view_user_page_contains_name(self):
        """ Resulting page should contain the referent user name """
        entry = dict(
            id = 9999,
            name = "fred",
            fullname = "Fred Nurk",
        )
        identity_url = "http://example.org/id/%(name)s" % entry
        server_url = "http://example.org/openidserver"
        page = pagetemplate.identity_view_user_page(
            entry, identity_url
        )
        page.values.update(dict(server_url=server_url))
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(id)s..." % entry, page_data
        )
        self.failUnlessOutputCheckerMatch(
            "...%(name)s..." % entry, page_data
        )
        self.failUnlessOutputCheckerMatch(
            "...%(fullname)s..." % entry, page_data
        )
        self.failUnlessOutputCheckerMatch(
            "...%(identity_url)s..." % locals(), page_data
        )

    def test_login_user_not_found_page_contains_name(self):
        """ Resulting page should contain the referent user name """
        name = "bogus"
        page = pagetemplate.login_user_not_found_page(name)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(name)s..." % locals(), page_data
        )

    def test_login_view_page_returns_page(self):
        """ View login should return page """
        page = pagetemplate.login_view_page()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...}", page_data
        )

    def test_login_cancel_returns_page(self):
        """ Cancel login should return page """
        page = pagetemplate.login_cancelled_page()
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "Page {...}", page_data
        )

    def test_login_submit_failed_page_contains_details(self):
        """ Resulting page should contain specified details """
        message = "Did gyre and gimble in the wabe"
        name = "fred"
        page = pagetemplate.login_submit_failed_page(message, name)
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(message)s..." % locals(), page_data
        )
        self.failUnlessOutputCheckerMatch(
            "...%(name)s..." % locals(), page_data
        )

    def test_wrong_authentication_page_contains_url(self):
        """ Resulting page should contain the specified URLs """
        want_username = "bill"
        want_id = "http://foo.example.org/id/bill"
        page = pagetemplate.wrong_authentication_page(
            want_username = want_username,
            want_id_url = want_id,
        )
        page_data = page.serialise()
        self.failUnlessOutputCheckerMatch(
            "...%(want_id)s..." % locals(), page_data
        )


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_gracied.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for gracied daemon module
"""

import sys
import os
from StringIO import StringIO

import scaffold
from scaffold import Mock
from test_httprequest import Stub_GracieServer

module_name = 'gracied'
module_file_under_test = os.path.join(scaffold.bin_dir, module_name)
gracied = scaffold.make_module_from_file(
    module_name, module_file_under_test
)


class Stub_GracieServer(object):
    """ Stub class for GracieServer """

    version = "3.14.test"

    def __init__(self, socket_params, opts):
        """ Set up a new instance """

    def serve_forever(self):
        pass


class Test_Gracie(scaffold.TestCase):
    """ Test cases for Gracie class """
    def setUp(self):
        """ Set up test fixtures """

        self.app_class = gracied.Gracie

        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.stub_server_class = Stub_GracieServer
        self.mock_server_class = Mock('GracieServer_class')
        self.mock_server_class.mock_returns = Mock('GracieServer')

        self.server_class_prev = gracied.GracieServer
        gracied.GracieServer = self.stub_server_class
        self.default_port_prev = gracied.default_port
        gracied.default_port = 7654

        self.valid_apps = {
            'simple': dict(
                args = dict(),
            ),
            'argv_loglevel_debug': dict(
                args = dict(),
                options = ["--log-level", "debug"],
            ),
            'change-host': dict(
                args = dict(),
                options = ["--host", "frobnitz"],
                host = "frobnitz",
            ),
            'change-port': dict(
                args = dict(),
                options = ["--port", "9779"],
                port = 9779,
            ),
            'change-address': dict(
                args = dict(),
                options = ["--host", "frobnitz", "--port", "9779"],
                host = "frobnitz",
                port = 9779,
            ),
            'change-root-url': dict(
                args = dict(),
                options = ["--root-url", "http://spudnik/spam"],
                root_url = "http://spudnik/spam",
            ),
            'change-address-and-root-url': dict(
                args = dict(),
                options = [
                    "--host", "frobnitz", "--port", "9779",
                    "--root-url", "http://spudnik/spam",
                ],
                host = "frobnitz",
                port = 9779,
                root_url = "http://frobnitz/spam",
            ),
        }

        for key, params in self.valid_apps.items():
            args = params['args']
            argv = []
            options = params.get('options', None)
            if options:
                argv.extend(options)
            args['argv'] = argv
            instance = self.app_class(**args)
            params['instance'] = instance

        self.iterate_params = scaffold.make_params_iterator(
            default_params_dict = self.valid_apps
        )

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        gracied.GracieServer = self.server_class_prev
        gracied.default_port = self.default_port_prev

    def test_instantiate(self):
        """ New Gracie instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_configures_logging(self):
        """ New Gracie instance should configure logging """
        params = self.valid_apps['simple']
        args = params['args']
        logging_prev = gracied.logging
        gracied.logging = Mock('logging')
        expect_stdout = """\
           Called logging.basicConfig(...)
           """
        instance = self.app_class(**args)
        gracied.logging = logging_prev
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_opts_version(self):
        """ Gracie instance should perform version action """

        argv = ["--version"]
        version_prev = gracied.__version__
        version_test = "Foo.Boo"
        gracied.__version__ = version_test
        expect_stdout = """\
            ...%(version)s...
            """ % dict(version=version_test)
        self.failUnlessRaises(SystemExit,
            self.app_class, argv=argv
        )
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )
        gracied.__version__ = version_prev

    def test_opts_help(self):
        """ Gracie instance should perform help action """
        argv = ["--help"]
        expect_stdout = """\
            usage: ...
            """
        gracied.GracieServer = self.mock_server_class
        self.failUnlessRaises(SystemExit,
            self.app_class, argv=argv
        )
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_opts_loglevel(self):
        """ Gracie instance should accept log-level setting """
        want_loglevel = "DEBUG"
        argv = ["--log-level", want_loglevel]
        instance = self.app_class(argv=argv)
        self.failUnlessEqual(want_loglevel, instance.opts.loglevel)

    def test_opts_datadir(self):
        """ Gracie instance should accept data-dir setting """
        want_dir = "/foo/bar"
        argv = ["--data-dir", want_dir]
        instance = self.app_class(argv=argv)
        self.failUnlessEqual(want_dir, instance.opts.datadir)

    def test_opts_host(self):
        """ Gracie instance should accept host setting """
        want_host = "frobnitz"
        argv = ["--host", want_host]
        instance = self.app_class(argv=argv)
        self.failUnlessEqual(want_host, instance.opts.host)

    def test_opts_port(self):
        """ Gracie instance should accept port setting """
        want_port = 9779
        argv = ["--port", want_port]
        instance = self.app_class(argv=argv)
        self.failUnlessEqual(want_port, instance.opts.port)

    def test_opts_root_url(self):
        """ Gracie instance should accept root_url setting """
        want_url = "http://spudnik/spam"
        argv = ["--root-url", want_url]
        instance = self.app_class(argv=argv)
        self.failUnlessEqual(want_url, instance.opts.root_url)

    def test_instantiates_server(self):
        """ Gracie instance should create a new server instance """
        params = self.valid_apps['simple']
        args = params['args']
        host = gracied.default_host
        port = gracied.default_port
        expect_stdout = """\
            Called GracieServer_class(
                (%(host)r, %(port)r),
                <Values ...>)
            ...""" % locals()
        gracied.GracieServer = self.mock_server_class
        instance = self.app_class(**args)
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )
        self.failUnless(instance.server is not None)

    def test_sets_specified_socket_params(self):
        """ Should set the server on the specified host:port """
        for key, params in self.iterate_params():
            args = params['args']
            default_address = (
                gracied.default_host, gracied.default_port
            )
            host = params.get('host', gracied.default_host)
            port = params.get('port', gracied.default_port)
            if (host, port) == default_address:
                continue
            expect_stdout = """\
                Called GracieServer_class(
                    (%(host)r, %(port)r),
                    <Values ...>)
                ...""" % locals()
            gracied.GracieServer = self.mock_server_class
            instance = self.app_class(**args)
            self.failUnlessOutputCheckerMatch(
                expect_stdout, self.stdout_test.getvalue()
            )
            self.stdout_test.truncate(0)

    def test_run_is_callable(self):
        """ Gracie.run should be callable """
        self.failUnless(callable(self.app_class.run))

    def test_run_calls_become_daemon(self):
        """ Gracie.run should attempt to become a daemon """
        params = self.valid_apps['simple']
        gracied.become_daemon = Mock('become_daemon')
        expect_stdout = """\
            Called become_daemon()
            """
        instance = params['instance']
        instance.run()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_run_starts_server(self):
        """ Gracie.run should start GracieServer if child fork """
        args = self.valid_apps['simple']['args']
        port = gracied.default_port
        expect_stdout = """\
            ...
            Called GracieServer.serve_forever()
            """
        gracied.GracieServer = self.mock_server_class
        instance = self.app_class(**args)
        instance.run()
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )


class Test_main(scaffold.TestCase):
    """ Test cases for __main__ function """

    def setUp(self):
        """ Set up test fixtures """
        mock_app_class = Mock('Gracie_class')
        mock_app_class.mock_returns = Mock('Gracie')
        self.mainfunc = gracied.__main__

        self.stdout_prev = sys.stdout
        self.stdout_test = StringIO()
        sys.stdout = self.stdout_test

        self.app_class_prev = gracied.Gracie
        gracied.Gracie = mock_app_class

    def tearDown(self):
        """ Tear down test fixtures """
        sys.stdout = self.stdout_prev
        gracied.Gracie = self.app_class_prev

    def test_is_callable(self):
        """ __main__ function should be callable """
        self.failUnless(callable(self.mainfunc))

    def test_instantiates_app_class(self):
        """ __main__() should instantiate the application class """
        args = dict()
        expect_stdout = """\
            Called Gracie_class(...)
            ..."""
        self.mainfunc(**args)
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_defaults_argv_to_sys_argv(self):
        """ __main__() should default commandline arguments to sys.argv """
        args = dict()
        expect_stdout = """\
            Called Gracie_class(%(argv)s)
            ...""" % dict(argv=sys.argv)
        self.mainfunc(**args)
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_passes_argv_to_server(self):
        """ __main__() should pass commandline arguments to application """
        argv = ["foo"]
        args = dict(
            argv=argv,
        )
        expect_stdout = """\
            Called Gracie_class(%(argv)s)
            ...""" % dict(argv=argv)
        self.mainfunc(**args)
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )

    def test_invokes_run(self):
        """ __main__() should invoke app's run() method """
        argv = ["foo"]
        args = dict(argv=argv)
        expect_stdout = """\
            Called Gracie_class(%(argv)s)
            Called Gracie.run(...)
        """ % dict(argv=argv)
        self.mainfunc(**args)
        self.failUnlessOutputCheckerMatch(
            expect_stdout, self.stdout_test.getvalue()
        )


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_session.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for session module
"""

import scaffold
from scaffold import Mock

from gracie import session


class Test_SessionManager(scaffold.TestCase):
    """ Test cases for SessionManager class """

    def setUp(self):
        """ Set up test fixtures """

        self.manager_class = session.SessionManager

    def test_create_session_should_return_session_id(self):
        """ Creating a session should return session ID """
        instance = self.manager_class()
        session_id = instance.create_session()
        self.failUnless(session_id is not None)

    def test_get_session_unknown_id_raises_keyerror(self):
        """ Getting an unknown session ID should raise KeyError """
        instance = self.manager_class()
        session_id = "DECAFBAD"
        self.failUnlessRaises(KeyError,
            instance.get_session, session_id
        )

    def test_get_session_returns_same_session(self):
        """ Getting a session by ID should return same username """
        instance = self.manager_class()
        session = dict(
            username = "fred",
            foo = "spam",
        )
        session_id = instance.create_session(session)
        session['session_id'] = session_id
        got_session = instance.get_session(session_id)
        self.failUnlessEqual(session, got_session)

    def test_create_session_should_create_unique_id(self):
        """ Creating a session should create unique ID each time """
        instance = self.manager_class()
        usernames = ["larry", "curly", "moe"]
        sessions = dict()
        for username in usernames:
            session_id = instance.create_session()
            self.failIf(session_id in sessions,
                "Session ID %(session_id)r already exists"
                " in %(sessions)r" % locals()
            )
            sessions[session_id] = dict(
                session_id = session_id,
                username = username,
            )

    def test_create_multiple_session_for_same_username(self):
        """ Creating multiple sessions for same username should succeed """
        instance = self.manager_class()
        usernames = ["larry", "curly", "moe"]
        sessions = dict()
        for username in usernames:
            for _ in range(10):
                session = dict(username=username)
                session_id = instance.create_session(session)
                session.update(dict(
                    session_id = session_id
                ))
                sessions[session_id] = session
        for session_id, session in sessions.items():
            got_session = instance.get_session(session_id)
            self.failUnlessEqual(session, got_session)

    def test_remove_session_unknown_should_raise_keyerror(self):
        """ Removing an unknown session ID should raise KeyError """
        instance = self.manager_class()
        session_id = "DECAFBAD"
        self.failUnlessRaises(KeyError,
            instance.remove_session, session_id
        )

    def test_remove_session_should_cause_get_session_failure(self):
        """ Removing a session should result in failure to get session """
        instance = self.manager_class()
        identity_name = "fred"
        session_id = instance.create_session()
        instance.remove_session(session_id)
        self.failUnlessRaises(KeyError,
            instance.get_session, session_id
        )


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



# -*- coding: utf-8 -*-

# pagetemplate.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Response page generation for OpenID provider
"""

from string import Template


page_template = Template("""\
<?xml version="1.0" encoding="$character_encoding" ?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" >
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
    <meta http-equiv="Content-Type"
        content="application/xhtml+xml; $character_encoding" />

    <title>$page_title</title>

    <style type="text/css">
    $css_sheet
    </style>

    $openid_metadata

</head>
$page_body
</html>
""")

body_template = Template("""\
<body>
$page_header
<div id="content">
<h1 id="title">$page_title</h1>
$page_content
</div><!-- content -->
$page_footer
</body>
""")

header_template = Template("""\
<div id="header">
<p id="banner"><a href="$root_url">Gracie</a></p>
$auth_section
</div><!-- banner -->
""")

footer_template = Template("""\
<div id="footer">
<p>Server version $server_version;
running on $server_location</p>
</div><!-- footer -->
""")

auth_section_template = Template("""\
<div id="auth-info">
<p><em>Status:</em> $login_status</p>
$logged_in_as
<p>$change_status</p>
</div><!-- auth-info -->
""")

css_sheet = """\
body {
    width: 100%;
    height: auto;
    margin: 0.0em;
    color: black;
    background-color: #FFB;
}

div#header {
    color: black;
    background-color: #FFC;
    height: 3.0em;
    margin: 0.0em;
    border: 0;
    border-bottom: 2px solid black;
    padding: 0.2em;
    padding-left: 1.0em;
}

div#footer {
    color: black;
    background-color: #FFC;
    border: 0;
    border-top: 1px solid black;
    border-bottom: 1px solid black;
    padding: 0.2em;
    font-size: 70%;
    font-family: sans-serif;
}

p#banner {
    width: 60%;
    float: left;
}

div#auth-info {
    float: right;
    color: black;
    background-color: #BCF;
    margin: 0.0em;
    border: 1px solid #88A;
    padding: 0.3em 0.5em;
    width: 30%;
    font-family: sans-serif;
    font-size: 70%;
    text-align: right;
}

div#content {
    margin: 0.0em;
    padding: 0.5em;
    padding-bottom: 3.0em;
}
"""


class Page(object):
    """ Web page """

    def __init__(self, title=None):
        """ Set up a new instance """
        self.character_encoding = "utf-8"
        self.title = title
        self.css_sheet = css_sheet
        self.openid_metadata = ""
        self.content = ""
        self.values = dict(
            server_version = None,
            server_location = None,
            auth_entry = None,
            root_url = None,
            server_url = None,
            login_url = None,
            logout_url = None,
        )

    def _get_auth_section(self, auth_entry):
        """ Get the authentication info section """
        login_url = self.values['login_url']
        logout_url = self.values['logout_url']
        login_status = "You are not logged in."
        logged_in_as = ""
        change_status = (
            """You may <a href="%(login_url)s">log in now</a>."""
            % locals()
        )
        if auth_entry:
            fullname = auth_entry['fullname']
            openid_url = self.values['openid_url']
            login_url = self.values['login_url']
            login_status = "You are logged in."
            logged_in_as = ("""\
                <a href="%(openid_url)s">%(openid_url)s</a>
                (%(fullname)s)
                """ % locals()
            )
            change_status = (
                """You may <a href="%(logout_url)s">log out now</a>."""
                % locals()
            )
        text = auth_section_template.substitute(locals())
        return text

    def _get_header(self):
        """ Get the page header """
        auth_section_text = self._get_auth_section(
            self.values['auth_entry'])
        text = header_template.substitute(self.values,
            auth_section=auth_section_text,
        )
        return text

    def _get_footer(self):
        """ Get the page footer """
        text = footer_template.substitute(self.values,
        )
        return text

    def serialise(self):
        """ Generate a text stream for page data """
        header_text = self._get_header()
        content_text = Template(self.content).substitute(self.values)
        footer_text = self._get_footer()
        body_text = body_template.substitute(self.values,
            page_title=self.title,
            page_header=header_text, page_footer=footer_text,
            page_content=content_text,
        )
        openid_metadata_text = Template(
            self.openid_metadata
        ).substitute(self.values)
        page_text = page_template.substitute(
            page_title=self.title, page_body=body_text,
            css_sheet=self.css_sheet,
            openid_metadata=openid_metadata_text,
            character_encoding=self.character_encoding,
        )
        return page_text


def internal_error_page(message):
    title = "Internal Server Error"
    page = Page(title)
    page.content = """
        <p>The server encountered an error trying to serve the request.
        The message was:</p>

        <pre>$message</pre>
    """
    page.values.update(dict(
        message = message,
    ))
    return page

def url_not_found_page(url):
    title = "Resource Not Found"
    page = Page(title)
    page.content = """
        <p>The requested resource was not found: $want_url</p>
    """
    page.values.update(dict(
        want_url = url,
    ))
    return page

def protocol_error_page(message):
    title = "Protocol Error"
    page = Page(title)
    page.content = """
        <p>The request did not conform to the expected protocol.
        The message was:</p>

        <pre>$message</pre>
    """
    page.values.update(dict(
        message = message,
    ))
    return page

def about_site_view_page():
    title = "About this site"
    page = Page(title)
    openid_project_url = "http://openid.net/"
    page.content = """
        <p>This is Gracie, an 
        <a href="%(openid_project_url)s">OpenID</a> provider.</p>

        <p>It provides OpenID identities for local accounts,
        and allows authentication against the local PAM system.</p>
    """ % locals()
    return page

def user_not_found_page(name):
    title = "User Not Found"
    page = Page(title)
    page.content = """
        <p>The requested user name does not exist: $user_name</p>
    """
    page.values.update(dict(
        user_name = name,
    ))
    return page

def identity_user_not_found_page(name):
    page = user_not_found_page(name)
    return page

def identity_view_user_page(entry, identity_url):
    title = "Identity page for %(fullname)s" % entry
    page = Page(title)
    page.openid_metadata = """
        <link rel="openid.server" href="$server_url" />
    """
    page.content = """
        <div id="identity-info">
        <table>
        <tr><th>OpenID</th><td><a href="$identity_url"
            >$identity_url</a></td></tr>
        <tr><th>User ID</th><td>$id</td></tr>
        <tr><th>Name</th><td>$name</td></tr>
        <tr><th>Full name</th><td>$fullname</td></tr>
        </table>
        </div><!-- identity-info -->
    """
    page.values.update(entry)
    page.values.update(dict(identity_url=identity_url))
    return page

def login_user_not_found_page(name):
    page = user_not_found_page(name)
    return page

def _login_form(message="", name=""):
    form_text = """
        <p class="message">$message</p>
        <form id="login" action="/login" method="POST">
        <p>
        <label for="username">Username</label>
        <input name="username" type="text" value="$username" />
        </p>
        <p>
        <label for="password">Password</label>
        <input name="password" type="password" />
        </p>
        <p>
        <input type="submit" name="submit" value="Sign in" />
        <input type="submit" name="cancel" value="Cancel" />
        </p>
        </form>
    """
    values = dict(message=message, username=name)
    form_text = Template(form_text).substitute(values)
    return form_text

def login_view_page():
    title = "Login"
    form_text = _login_form()
    page = Page(title)
    page.values.update(dict(form=form_text))
    page.content = """
        $form
    """
    return page

def login_cancelled_page():
    title = "Login Cancelled"
    page = Page(title)
    page.content = """
        <p>The login was cancelled.</p>
        <p>You can <a href="$login_url">log in now</a> if you want.</p>
    """
    return page

def login_submit_failed_page(message, name):
    title = "Login Failed"
    page = Page(title)
    form_text = _login_form(message, name)
    page.values.update(dict(form=form_text))
    page.content = """
        $form
    """
    return page

def wrong_authentication_page(want_username, want_id_url):
    title = "Authentication Required"
    page = Page(title)
    message_template = Template("""
        The requested action can only be performed if you log in
        as the identity <a href="$want_id">$want_id</a>
    """)
    message = message_template.substitute(dict(
        want_username = want_username,
        want_id = want_id_url,
    ))
    form_text = _login_form(message, want_username)
    page.values.update(dict(
        form = form_text,
    ))
    page.content = """
        $form
    """
    return page



# (c) 2006 Ian Bicking and contributors
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php

r"""
minimock is a simple library for doing Mock objects with doctest.
When using doctest, mock objects can be very simple.

Here's an example of something we might test, a simple email sender::

    >>> import smtplib
    >>> def send_email(from_addr, to_addr, subject, body):
    ...     conn = smtplib.SMTP('localhost')
    ...     msg = 'To: %s\nFrom: %s\nSubject: %s\n\n%s' % (
    ...         to_addr, from_addr, subject, body)
    ...     conn.sendmail(from_addr, [to_addr], msg)
    ...     conn.quit()

Now we want to make a mock ``smtplib.SMTP`` object.  We'll have to
inject our mock into the ``smtplib`` module::

    >>> smtplib.SMTP = Mock('smtplib.SMTP')
    >>> smtplib.SMTP.mock_returns = Mock('smtp_connection')

Now we do the test::

    >>> send_email('ianb@colorstudy.com', 'joe@example.com',
    ...            'Hi there!', 'How is it going?')
    Called smtplib.SMTP('localhost')
    Called smtp_connection.sendmail(
        'ianb@colorstudy.com',
        ['joe@example.com'],
        'To: joe@example.com\nFrom: ianb@colorstudy.com\nSubject: Hi there!\n\nHow is it going?')
    Called smtp_connection.quit()

Voila!  We've tested implicitly that no unexpected methods were called
on the object.  We've also tested the arguments that the mock object
got.  We've provided fake return calls (for the ``smtplib.SMTP()``
constructor).  These are all the core parts of a mock library.  The
implementation is simple because most of the work is done by doctest.
"""

class Mock(object):

    def __init__(self, name):
        self.mock_name = name
        self.mock_returns = None
        self.mock_attrs = {}

    def __repr__(self):
        return '<Mock %s %s>' % (hex(id(self)), self.mock_name)

    def __call__(self, *args, **kw):
        parts = [repr(a) for a in args]
        parts.extend(
            '%s=%r' % (items) for items in sorted(kw.items()))
        msg = 'Called %s(%s)' % (self.mock_name, ', '.join(parts))
        if len(msg) > 80:
            msg = 'Called %s(\n    %s)' % (
                self.mock_name, ',\n    '.join(parts))
        print msg
        return self.mock_returns

    def __getattr__(self, attr):
        if attr not in self.mock_attrs:
            if self.mock_name:
                new_name = self.mock_name + '.' + attr
            else:
                new_name = attr
            self.mock_attrs[attr] = Mock(new_name)
        return self.mock_attrs[attr]

    
if __name__ == '__main__':
    import doctest
    doctest.testmod()
    



# -*- coding: utf-8 -*-

# scaffold.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Scaffolding for unit test modules
"""

import unittest
import doctest
import logging
import os
import sys
import textwrap
from minimock import Mock

test_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(test_dir)
if not test_dir in sys.path:
    sys.path.insert(1, test_dir)
if not parent_dir in sys.path:
    sys.path.insert(1, parent_dir)
bin_dir = os.path.join(parent_dir, "bin")

# Disable all but the most critical logging messages
logging.disable(logging.CRITICAL)


def suite(module_name):
    """ Create the test suite for named module """
    from sys import modules
    loader = unittest.TestLoader()
    suite = loader.loadTestsFromModule(modules[module_name])
    return suite

def unittest_main(argv=None):
    """ Mainline function for each unit test module """

    from sys import argv as sys_argv
    if not argv:
        argv = sys_argv

    exitcode = None
    try:
        unittest.main(argv=argv, defaultTest='suite')
    except SystemExit, e:
        exitcode = e.code

    return exitcode


def make_module_from_file(module_name, file_name):
    """ Make a new module object from the code in specified file """

    from types import ModuleType
    module = ModuleType(module_name)

    module_file = open(file_name, 'r')
    exec module_file in module.__dict__

    return module


def make_params_iterator(default_params_dict):
    """ Make a function for generating test parameters """

    def iterate_params(params_dict=None):
        """ Iterate a single test for a set of parameters """
        if not params_dict:
            params_dict = default_params_dict
        for key, params in params_dict.items():
            yield key, params

    return iterate_params


class TestCase(unittest.TestCase):
    """ Test case behaviour """

    def failUnlessOutputCheckerMatch(self, want, got):
        """ Fail the test unless output matches via doctest OutputChecker """
        checker = doctest.OutputChecker()
        want = textwrap.dedent(want)
        got = textwrap.dedent(got)
        self.failUnlessEqual(True,
            checker.check_output(want, got, doctest.ELLIPSIS),
            "Expected %(want)r, got %(got)r:"
            "\n--- want: ---\n%(want)s"
            "\n--- got: ---\n%(got)s" % locals()
        )


class Test_Exception(TestCase):
    """ Test cases for exception classes """

    def __init__(self, *args, **kwargs):
        """ Set up a new instance """
        self.valid_exceptions = NotImplemented
        super(Test_Exception, self).__init__(*args, **kwargs)

    def setUp(self):
        """ Set up test fixtures """
        for exc_type, params in self.valid_exceptions.items():
            args = (None,) * params['min_args']
            instance = exc_type(*args)
            self.valid_exceptions[exc_type]['instance'] = instance

        self.iterate_params = make_params_iterator(
            default_params_dict = self.valid_exceptions
        )

    def test_exception_instance(self):
        """ Exception instance should be created """
        for key, params in self.iterate_params():
            instance = params['instance']
            self.failUnless(instance is not None)

    def test_exception_types(self):
        """ Exception instances should match expected types """
        for key, params in self.iterate_params():
            instance = params['instance']
            for match_type in params['types']:
                match_type_name = match_type.__name__
                self.failUnless(isinstance(instance, match_type),
                    "%(instance)r is not an instance of %(match_type_name)s"
                        % locals()
                )



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# setup.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Package setup script
"""

import ez_setup
ez_setup.use_setuptools()

import sys
import os
import stat

from setuptools import setup, find_packages
from setuptools import command as st_command
from setuptools.command.install_scripts import install_scripts

main_module_name = 'gracie'
main_module = __import__(main_module_name)

description = main_module.__doc__.split('\n\n', 1)


class gracie_install_scripts(install_scripts):
    """ Custom install_scripts command for Gracie """

    def run(self):
        """ Run the command """
        self.mkpath(os.path.normpath(self.install_dir))

        self.outfiles = []
        _p = os.path.join
        scripts = {
            _p(self.build_dir, "gracied"): self.install_dir,
        }
        for src_file, dest_dir in scripts.items():
            ofile, copied = self.copy_file(src_file, dest_dir)
            if copied:
                self.outfiles.append(ofile)

        if os.name == "posix":
            for ofile in self.get_outputs():
                if not self.dry_run:
                    mode = os.stat(ofile)[stat.ST_MODE]
                    new_mode = (mode | 0555) & 07777
                    os.chmod(ofile, new_mode)


setup(
    name = main_module_name,
    version = main_module.__version__,
    packages = find_packages(
        exclude = ['test'],
    ),
    cmdclass = {
        'install_scripts': gracie_install_scripts,
    },

    # setuptools metadata
    zip_safe = True,
    test_suite = "test.suite.suite",
    install_requires = [
        "python-openid >= 1.2",
        "Routes >= 1.6.3",
    ],

    # PyPI metadata
    author = main_module.__author_name__,
    author_email = main_module.__author_email__,
    description = description[0].strip(),
    license = main_module.__license__,
    keywords = "gracie openid identity authentication provider",
    url = main_module.__url__,
    long_description = description[1],
    classifiers = [
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Topic :: Internet",
        "Topic :: System",
        "Operating System :: POSIX",
        "Intended Audience :: System Administrators",
    ],
)




#! /usr/bin/env python
# -*- coding: utf-8 -*-

# suite.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test suite for the application modules
"""

import unittest
import sys
import os

import scaffold


def get_python_modules(file_list, file_suffix = '.py'):
    """ Return a list of module names from a filename list """
    python_modules = [m[:m.rfind(file_suffix)] for m in file_list
        if m.endswith(file_suffix)]
    return python_modules

def get_test_modules(module_list, module_prefix = 'test_'):
    """ Return the list of modules that are named as test modules """
    test_modules = [m for m in module_list
        if m.startswith(module_prefix)]
    return test_modules


def suite():
    """ Create the test suite for this module """
    loader = unittest.TestLoader()
    python_modules = get_python_modules(os.listdir(scaffold.test_dir))
    module_list = get_test_modules(python_modules)
    suite = loader.loadTestsFromNames(module_list)

    return suite


def __main__(argv=None):
    """ Mainline function for this module """
    from sys import argv as sys_argv
    if not argv:
        argv = sys_argv

    exitcode = None
    try:
        unittest.main(argv=argv, defaultTest='suite')
    except SystemExit, e:
        exitcode = e.code

    return exitcode

if __name__ == '__main__':
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

# test_authorisation.py
# Part of Gracie, an OpenID provider
#
# Copyright © 2007 Ben Finney <ben+python@benfinney.id.au>
# This is free software; you may copy, modify and/or distribute this work
# under the terms of the GNU General Public License, version 2 or later.
# No warranty expressed or implied. See the file LICENSE for details.

""" Unit test for authorisation module
"""

import scaffold
from scaffold import Mock

from gracie import authorisation


class Test_ConsumerAuthStore(scaffold.TestCase):
    """ Test cases for ConsumerAuthStore class """

    def setUp(self):
        """ Set up test fixtures """
        self.store_class = authorisation.ConsumerAuthStore

    def test_instantiate(self):
        """ New ConsumerAuthStore instance should be created """
        instance = self.store_class()
        self.failUnless(instance is not None)

    def test_is_authorised_unknown_returns_false(self):
        """ is_authorised for unknown args should return False """
        instance = self.store_class()
        auth_tuple = ("bogus", "bogus")
        is_authorised = instance.is_authorised(auth_tuple)
        self.failUnlessEqual(False, is_authorised)

    def test_store_authorisation_result_in_authorisation(self):
        """ store_authorisation should let is_authorised succeed """
        instance = self.store_class()
        identity = "/id/fred"
        trust_root = "http://example.com/"
        auth_tuple = (identity, trust_root)
        status = True
        instance.store_authorisation(auth_tuple, status)
        got_status = instance.is_authorised(auth_tuple)
        self.failUnlessEqual(True, got_status)

    def test_remove_authorisation_unknown_should_succeed(self):
        """ remove_authorisation for unknown args should succeed """
        instance = self.store_class()
        auth_tuple = ("bogus", "bogus")
        instance.remove_authorisation(auth_tuple)
        is_authorised = instance.is_authorised(auth_tuple)
        self.failUnlessEqual(False, is_authorised)

    def test_remove_authorisation_result_in_no_authorisation(self):
        """ remove_authorisation should make is_authorised return False """
        instance = self.store_class()
        identity = "/id/fred"
        trust_root = "http://example.com/"
        auth_tuple = (identity, trust_root)
        status = True
        instance.store_authorisation(auth_tuple, status)
        instance.remove_authorisation(auth_tuple)
        got_status = instance.is_authorised(auth_tuple)
        self.failUnlessEqual(False, got_status)


suite = scaffold.suite(__name__)

__main__ = scaffold.unittest_main

if __name__ == '__main__':
    import sys
    exitcode = __main__(sys.argv)
    sys.exit(exitcode)
